/*
 * Copyright (C) 2021 Associated Universities, Inc. Washington DC, USA.
 *
 * This file is part of NRAO Workspaces.
 *
 * Workspaces is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Workspaces is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Workspaces.  If not, see <https://www.gnu.org/licenses/>.
 */
import {Component, OnDestroy, OnInit} from '@angular/core';
import {verticleSlide} from "../../animations";
import {ResultType} from "../model/result-type";
import {Subject, Subscription} from "rxjs";
import {AppConfigService} from "../../env/app-config.service";
import {FacetsService} from "../services/facets.service";
import {ResultTypeService} from "../services/result-type.service";
import {SelectedFilterService} from "../services/selected-filter.service";
import {ActivatedRoute, NavigationEnd, Router} from "@angular/router";
import {SearchResultsService} from "../services/search-results.service";
import {WebAnalyticsService} from "../../web-analytics/web-analytics.service";
import {Field} from "../model/field";

@Component({
  selector: 'app-search',
  templateUrl: './search.component.html',
  styleUrls: ['./search.component.scss'],
  animations: [verticleSlide]

})
export class SearchComponent implements OnInit, OnDestroy {

  private ngUnsubscribe = new Subject();

  public showFilterList = true;
  public spinner: boolean = true;
  public spinnerText: string = 'Loading';

  public resultTypes = ResultType.getResultTypes(); // to create the resultType links
  public resultType: ResultType; // the active result type
  private resultTypeSub: Subscription; // subscription to the active result type

  public config: any;
  private configSub: Subscription;
  public facets: any;
  private facetSub: Subscription;

  public allowedResultsPerPage: Array<number> = [25,50,100,200];
  public resultsPerPage: number = 25;
  public currentPage: number = 1;
  public searchResults: Array<any> = [];
  public numResults: number = 0;
  public pages: number = 1;

  constructor(
    private appConfig: AppConfigService,
    private facetService: FacetsService,
    private resultTypeService: ResultTypeService,
    private selectedFilterService: SelectedFilterService,
    private router: Router,
    private route: ActivatedRoute,
    private searchResultsService: SearchResultsService,
    private analytics: WebAnalyticsService){

    this.router.events.subscribe((ev) => {
      if (ev instanceof NavigationEnd) {
        let resultType = this.route.snapshot.firstChild.url[0].path;
        if (!this.resultType || resultType !== this.resultType.name) {
          this.spinner = true;
          this.spinnerText = 'Loading Search Type';
          this.resultTypeService.updateResultType(resultType);
        } else if (this.resultType){
          this.loadUrlParams();
        }
        this.analytics.sendBeacon();
      }
    });
    this.spinnerText = 'Configuring';
    this.loadConfig();

  }

  ngOnInit(): void {
  }

  loadConfig(){
    this.configSub = this.appConfig.getConfig().subscribe((config:string) => {
      this.config = JSON.parse(config);
      this.spinnerText = 'Getting Filter Options';
      this.loadFacets();
    });
  }

  loadFacets(){
    this.facetSub = this.appConfig.getFacets().subscribe((facets:string) => {
      this.facets = JSON.parse(facets).facet_list;
      this.facetService.setFacets(this.facets);
      this.spinnerText = 'Loading Search Type';
      this.loadResultType();
    });
  }

  loadResultType(){
    this.resultTypeSub = this.resultTypeService.resultType$.subscribe( resultType => {
      this.resultType = resultType;
      this.spinner = false;
      this.spinnerText = 'Loading';
      this.loadUrlParams();
    });
  }

  loadUrlParams(){
    // check the queryParams
    let queryParams = this.route.snapshot.queryParamMap;
    let fields = Field.getFields();
    // iterate over the queryParams
    for (let qp of queryParams.keys) {
      qp = qp.toLowerCase();
      if (fields.indexOf(qp) !== -1) { // make sure it's a valid field
        let field = Field.getField(qp);
        if (field.allowMultipleValues) {
          let qpv = queryParams.getAll(qp);
          if (qpv && qpv.length > 0) { // first make sure we have anything
            if (this.facets.hasOwnProperty(field.name)) { // get the valid values for this field
              let facets = this.facets[field.name]; // to test for valid value
              for (let f of facets) {
                for (let aVal of qpv) {
                  if (f.name.toLowerCase() == aVal.toLowerCase()) {
                    this.selectedFilterService.addSelectedFilter(field.label, field.name, f.name);
                  }
                }
              }
            } else {
              console.warn('No facets found for ' + field.name);
            }
          }
        } else {
          let qpv = queryParams.get(qp);
          if (qpv && qpv.length > 0) { // make sure we have something
            this.selectedFilterService.addReplaceSelectedFilter(field.label, field.name, qpv);
          }
        }
      }
    }
    this.selectedFilterService.sortSelectedFilters();
    this.getSearchResults();
  }

  setResultsPerPage(rows: number){
    if(this.allowedResultsPerPage.indexOf(rows) == -1){
      return false;
    }
    this.resultsPerPage = rows;
    this.pages = Math.ceil(this.numResults / this.resultsPerPage);
    this.currentPage = 1;
    this.getSearchResults();
  }

  goToPage(page: number){
    if(page < 1){ page = 1; }
    if(page > this.pages){ page = this.pages; }
    if(this.currentPage == page){
      return false;
    }
    this.currentPage = page;
    this.getSearchResults();
    return true;
  }

  getPages(){
    return new Array(this.pages);
  }

  getSearchResults(){
    this.searchResults = [];
    this.pages = 1;
    this.spinnerText = 'Getting Search Results';
    this.spinner = true;
    this.searchResultsService.getResults(this.resultType.resultsEndPoint, this.resultsPerPage * (this.currentPage - 1), this.resultsPerPage).subscribe((searchResults) => {
      console.log(searchResults);
      const keys = searchResults.headers.keys();
      let headers = keys.map(key => `${key}: ${searchResults.headers.get(key)}`);
      let jsonSearchResults = JSON.parse(searchResults.body);
      console.log(jsonSearchResults.data[0]);
      this.searchResults = jsonSearchResults.data;
      this.numResults = jsonSearchResults.n_results;
      this.pages = Math.ceil(this.numResults / this.resultsPerPage);
      this.spinner = false;
      this.spinnerText = 'Loading';
    });
  }

  ngOnDestroy(){
    this.ngUnsubscribe.next();
    this.ngUnsubscribe.complete();
  }

}

